var fs = require('fs-extra');
var path = require('path');
var gulp = require('gulp');
var sequence = require('gulp-sequence');
var stylus = require('gulp-stylus');
var clean = require('gulp-clean');
var webpack = require('webpack');
var rename = require('gulp-rename');
var argv = require('yargs').argv;
var glob = require('glob');
var shell = require('shelljs');
var gulpif = require('gulp-if');
var useref = require('gulp-useref');
var packager = require('electron-packager');
var VectrApp = require('electron-connect').server.create();

var packageJSON = require('./package.json');
var webpackConfig = require('./webpack.config');

var CURRENT_ENVIRONMENT = 'development';
var finalAppPaths = [];

function isProduction() {
  return CURRENT_ENVIRONMENT === 'production';
}

gulp.task('cleanup', function() {
  var noCleanup = argv.nocleanup;
  if (noCleanup) {
    // do nothing
  }
  else {
    return gulp
      .src(['./out'], {
        read: false
      })
      .pipe(clean());
  }
});

gulp.task('html', function() {
  var assets = useref.assets();

  return gulp
    .src('./_index.html')
    .pipe(gulpif(isProduction(), assets))
    .pipe(gulpif(isProduction(), assets.restore()))
    .pipe(gulpif(isProduction(), useref()))
    .pipe(rename('index.html'))
    .pipe(gulp.dest('./'));
});

gulp.task('stylus', function() {
  return gulp
    .src(['./src/public/stylus/**/*.styl'])
    .pipe(stylus())
    .pipe(gulp.dest('./src/public/css'))
});

gulp.task('webpack', function(done) {
  webpack(webpackConfig, function(error) {
    if (error) {
      throw error;
    }
    else {
      done();
    }
  });
});

gulp.task('package', function(done) {
  var packagerOptions = {};
  var devDependencies = packageJSON.devDependencies;
  var devDependenciesKeys = Object.keys(devDependencies);
  var ignoreFiles = [
    '^/out/',
  ];

  devDependenciesKeys.forEach(function(key) {
    ignoreFiles.push('^/node_modules/' + key);
  });

  var arch = process.arch || 'ia32';
  var platform = argv.platform || process.platform;
  var sign = argv.sign;

  var outputPlatformName;
  platform = platform.toLowerCase();
  switch (platform) {
    case 'mac':
    case 'darwin':
      outputPlatformName = 'mac';
      platform = 'darwin';
      arch = 'x64';
      break;
    case 'win':
    case 'win32':
    case 'windows':
      outputPlatformName = 'windows';
      platform = 'win32';
      arch = 'ia32';
      break;
    case 'linux64':
      outputPlatformName = 'linux';
      platform = 'linux';
      arch = 'x64';
      break;
    case 'linux32':
      outputPlatformName = 'linux';
      platform = 'linux';
      arch = 'ia32';
      break;
    default:
      console.log('We don\'t support your platform ' + platform);
      process.exit(1);
      break;
  }

  var iconFolderPath = './src/public/images/icons';
  var iconPath;

  if (platform === 'darwin') {
    iconPath = path.join(iconFolderPath, 'vectr.icns');
  }
  else {
    iconPath = path.join(iconFolderPath, 'vectr.ico');
  }

  var ignorePath = ignoreFiles.join('|');
  var ignoreRegexp = new RegExp(ignorePath, 'ig');

  // required options
  packagerOptions['dir'] = './';
  packagerOptions['platform'] = platform;
  packagerOptions['arch'] = arch;

  // all platform options
  packagerOptions['app-copyright'] = 'Vectr';
  packagerOptions['app-version'] = packageJSON.version;
  packagerOptions['build-version'] = packageJSON.version;
  packagerOptions['icon'] = iconPath;
  packagerOptions['ignore'] = ignoreRegexp;
  packagerOptions['name'] = 'Vectr';
  packagerOptions['out'] = './out';
  packagerOptions['overwrite'] = true;
  packagerOptions['version'] = '0.33.8';

  // darwin specific options
  packagerOptions['app-bundle-id'] = 'vectr';
  packagerOptions['app-category-type'] = 'public.app-category.graphics-design';
  packagerOptions['helper-bundle-id'] = 'vectr';

  if (sign) {
    packagerOptions['osx-sign'] = {
      'identity': sign
    };
  }

  // win32 specific options
  packagerOptions['version-string'] = {
    'CompanyName': 'Vectr',
    'LegalCopyright': 'No',
    'FileDescription': 'Vectr - Free Design App For Web + Desktop',
    'FileVersion': packageJSON.version,
    'ProductVersion': packageJSON.version,
    'ProductName': 'Vectr',
    'InternalName': 'Vectr'
  };

  packager(packagerOptions, function(error, appPaths) {
    if (error) {
      console.log(error);
      process.exit(1);
    }
    else {
      // TODO
      // we should support to build all platforms at once later !
      // something like [ 'build/vectr-darwin-x64' ]
      finalAppPaths = appPaths;
      done();
    }
  });
});

gulp.task('license', function(done) {
  var currentLicenseFile = path.join(__dirname, 'LICENSE');

  var promises = finalAppPaths.map(function(appPath) {
    var targetLicenseFile = path.join(appPath, 'LICENSE');
    var promise = new Promise(function(resolve, reject) {
      fs.copy(currentLicenseFile, targetLicenseFile, function(error) {
        if (error) {
          reject(error);
        }
        else {
          resolve();
        }
      });
    });
    return promise;
  });

  Promise.all(promises).then(function() {
    done();
  }).catch(function(error) {
    console.log(error)
    process.exit(1);
  });
});

gulp.task('env', function(done) {
  var command = shell.exec('git log --format="%H" -n 1', {
    silent: true
  });
  var commitId = 'unknown';
  if (command.code === 0) {
    // take off ending new line
    commitId = command.output.replace(/\r|\n/, '');
  }

  var envInfo = {
    env: CURRENT_ENVIRONMENT,
    commitId: commitId
  };

  fs.writeFile('env.json', JSON.stringify(envInfo), done);
});

gulp.task('watch', function() {
  VectrApp.start();

  shell.exec('webpack --watch', {
    async: true
  });

  gulp.watch([
    './index.html',
    './src/public/css/**',
    './vectr.bundled.js'
  ], VectrApp.reload);

  gulp.watch([
    './src/public/stylus/**/*.styl'
  ], ['stylus']);
});

gulp.task('default', function(done) {
  CURRENT_ENVIRONMENT = 'development';
  sequence(
    'env',
    'stylus',
    'html',
    'webpack'
  )(done);
});

gulp.task('build', function(done) {
  CURRENT_ENVIRONMENT = 'production';
  sequence(
    'cleanup',
    'env',
    'stylus',
    'html',
    'webpack',
    'package',
    'license'
  )(done);
});

gulp.task('build:all', function() {
  // TODO
  // move to internal gulp tasks later
  shell.exec('gulp build --nocleanup --platform=mac --sign=7R4WM3WCNR');
  shell.exec('gulp build --nocleanup --platform=win');
  shell.exec('gulp build --nocleanup --platform=linux32');
  shell.exec('gulp build --nocleanup --platform=linux64');
  shell.exec('gulp dist:zip');
});

gulp.task('dist:zip', function(done) {
  // all zip files will be kept in out/*
  shell.exec('mkdir -p out');

  glob('out/*', function (err, subdirs) {
    if (err) {
      throw err;
    }

    subdirs.forEach(function(subdir) {
      var subdirStats = fs.statSync(subdir);
      if (!subdirStats.isDirectory()) {
        return;
      }

      var outputPath = subdir + '-' + packageJSON.version + '.zip';
      outputPath = outputPath.replace('build', 'out');

      // Like what we are doing when using right-click zip
      var command =
        'ditto -c -k --sequesterRsrc --keepParent ' + subdir + ' ' + outputPath;
      var output = shell.exec(command);
      if (output.code !== 0) {
        throw new Error('can\'t zip ' + subdir);
      }
    });

    done();
  });
});
